<?php

/**
 * Ajax handler
 * 
 * @package
 */

namespace WPFunnels\Ajax_Handler;

use Elementor\Plugin;
use Elementor\Utils;
use WPFunnels\Frontend\Recaptcha\Google_Recaptcha_Handler;
use WPFunnels\Optin\Optin_Record;
use WPFunnels\Wpfnl_functions;
use function Mollie\WooCommerce\errorNotice;
use WPFunnels\Conditions\Wpfnl_Condition_Checker;

/**
 * Class Ajax_Handler
 *
 * @package WPFunnels\Widgets\Elementor
 */
class Ajax_Handler
{

	public function __construct()
	{
		add_action('wp_ajax_wpfnl_optin_submission', [$this, 'optin_form_submission']);
		add_action('wp_ajax_nopriv_wpfnl_optin_submission', [$this, 'optin_form_submission']);

		add_action('wp_ajax_wpfnl_shortcode_optin_submission', [$this, 'wpfnl_shortcode_optin_submission']);
		add_action('wp_ajax_nopriv_wpfnl_shortcode_optin_submission', [$this, 'wpfnl_shortcode_optin_submission']);

		add_action('wp_ajax_wpfnl_gutenberg_optin_submission', [$this, 'gutenberg_optin_form_submission']);
		add_action('wp_ajax_nopriv_wpfnl_gutenberg_optin_submission', [$this, 'gutenberg_optin_form_submission']);
	}


	/**
	 * Optin form submission handler for elementor
	 * 
	 * @return void
	 * @since  2.5.7
	 */
	public function optin_form_submission()
	{
		check_ajax_referer('optin_form_nonce', 'security');

		$step_id 	= isset($_POST['step_id']) ? $_POST['step_id'] : '';
		$postData 	= isset($_POST['postData']) ? $_POST['postData'] : '';
		
		$funnel_id	= Wpfnl_functions::get_funnel_id_from_step($step_id);
		$post_data  = array();
		parse_str($postData, $post_data);

		if (isset($post_data['email'])){
			$post_data['email'] = strtolower($post_data['email']);
		}
		
		$url 	= isset($_POST['url']) ? $_POST['url'] : '';
		$parts = parse_url($url);
		$query = [];
		if( isset($parts['query']) ){
			parse_str($parts['query'], $query);
		}
		
		$elementor 	= Plugin::instance();
		$document 	= $elementor->documents->get($step_id);
		$form 		= null;
		if ($document) {
			$form = Utils::find_element_recursive($document->get_elements_data(), $post_data['form_id']);
		}

		if (empty($form)) {
			$results = array(
				'message'	=> 'invalid_form',
				'success'	=> false
			);
			echo json_encode($results);
			die();
		}

		$widget 			= $elementor->elements_manager->create_element_instance($form);
		$form['settings'] 	= $widget->get_settings_for_display();
		unset($post_data['post_id']);
		unset($post_data['form_id']);

		$record 			= new Optin_Record($post_data, $form);
		$fields				= $record->get_fields();
		$name				= '';
		if ($fields) {
			foreach ($fields as $key => $value) {
				if ('email' === $key) {
					$name = strstr($value, '@', true);;
				} elseif ('last_name' === $key) {
					$name = $value;
				} elseif ('first_name' === $key) {
					$name = $value;
				}
			}
		}
	
		$response 	= array(
			'success'			=> true,
			'post_action' 		=> 'notification',
			'notification_text' => '',
			'redirect_url'		=> '#'
		);

		$post_action 					= $form['settings']['post_action'];
		$response['post_action'] 		= $post_action;
		$action_type 					= '';
		$response['notification_text'] 	= isset($form['settings']['notification_text']) && $form['settings']['notification_text'] ? $form['settings']['notification_text'] : '';
		$next_step = [];
		switch ($post_action) {
			case 'notification':
				$response['redirect']	= false;
				break;
			case 'redirect_to':
				$action_type 			= 'redirect_to_url';
				$response['redirect']	= true;
				if (!empty($form['settings']['redirect_url'])) {
					$response['redirect_url'] 	= add_query_arg( $this->prepare_query_param($name,$query), $form['settings']['redirect_url']['url']);
				} else {
					$response['redirect_url'] = '#';
				}
				break;
			default:
				$action_type 				= 'next_step';
				$next_step 					= Wpfnl_functions::get_next_step($funnel_id, $step_id);
				$response['redirect_url'] 	= add_query_arg( $this->prepare_query_param($name,$query), get_the_permalink($next_step['step_id']));
				$response['redirect']		= true;
		}

		$admin_email 	 		= isset($form['settings']['admin_email']) ? $form['settings']['admin_email'] : false;
		$admin_email_subject 	= isset($form['settings']['admin_email_subject']) ? $form['settings']['admin_email_subject'] : 'Opt-in form Submission';

		$user_info = $record->form_data;
		
		/**
		 * User registration
		 */
		if (!empty($user_info['optin_allow_registration']) && $user_info['optin_allow_registration'] == 'yes') {

			$this->create_user_optin_allow_registration($name, $user_info);
		}


		/**
		 * Recaptcha Check
		 */
		$is_recaptcha 			= isset($post_data['wpf-is-recapcha']) ? $post_data['wpf-is-recapcha'] : 'no';
		$recaptcha_site_key 	= isset($post_data['wpf-optin-g-token']) ? $post_data['wpf-optin-g-token'] : '';
		$recaptcha_site_secrect = isset($post_data['wpf-optin-g-secret-key']) ? $post_data['wpf-optin-g-secret-key'] : '';

		$response = $this->send_mail_after_checking_google_recaptcha_response($is_recaptcha, $recaptcha_site_key, $recaptcha_site_secrect, $admin_email, $admin_email_subject, $user_info, $response);


		/**
		 * Submit & process form data
		 */

		do_action('wpfunnels/after_optin_submit', $step_id, $post_action, $action_type, $record);

		if (!empty($next_step['step_type']) && 'conditional' ==  $next_step['step_type']) {
			$next_step = $this->match_conditional_step($next_step, $funnel_id, $step_id );
			$response['redirect_url'] 	= add_query_arg( $this->prepare_query_param($name,$query), get_the_permalink($next_step['step_id']));
		}

		echo json_encode($response, true);
		die();
	}


	/**
	 * Prepare query param for redirect url
	 * 
	 * @param String $name
	 * @param Array  $query
	 * 
	 * @return Array
	 * @since  2.5.9
	 */
	private function prepare_query_param( $name, $query ){
		$query_param['optin'] = true;
		$query_param['uname'] = $name;
		if( isset($query['wpfnl-order'], $query['wpfnl-key']) ){
			$query_param['wpfnl-order'] = $query['wpfnl-order'];
			$query_param['wpfnl-key'] = $query['wpfnl-key'];
		}
		return $query_param;
	}



	/**
	 * Optin submission handling for shortcode
     *
	 * @since  @since 2.5.7
	 * @return void
	 */
	public function wpfnl_shortcode_optin_submission()
	{
		check_ajax_referer('optin_form_nonce', 'security');
		$response 	= array(
			'success'			=> true,
			'post_action' 		=> 'notification',
			'notification_text' => '',
			'redirect_url'		=> '#'
		);
		$postData 	= isset($_POST['postData']) ? $_POST['postData'] : '';
		parse_str($postData, $post_data);

		if (isset($post_data['email'])){
			$post_data['email'] = strtolower($post_data['email']);
		}

		$url 	= isset($_POST['url']) ? $_POST['url'] : '';
		$parts = parse_url($url);
		$query = [];
		if( isset($parts['query']) ){
			parse_str($parts['query'], $query);
		}

		$step_id    = isset($post_data['post_id']) ? $post_data['post_id'] : '';
		$funnel_id = '';
		if ($step_id) {
			$funnel_id	= Wpfnl_functions::get_funnel_id_from_step($step_id);
		}

		$post_action 					= isset($post_data['post_action']) && $post_data['post_action'] ? $post_data['post_action'] : 'notification';
		$response['post_action'] 		= $post_action;
		$action_type 					= '';
		$response['notification_text'] 	= isset($post_data['notification_text']) && $post_data['notification_text'] ? $post_data['notification_text'] : '';
		$user_info = array();
		if ($post_data) {
			foreach ($post_data as $key => $value) {
				if ('email' === $key) {
					$name = strstr($value, '@', true);
					$user_info['email'] = $value;
				} elseif ('last_name' === $key) {
					$name = $value;
					$user_info['last_name'] = $value;
				} elseif ('first_name' === $key) {
					$name = $value;
					$user_info['first_name'] = $value;
				} elseif ('phone' === $key) {
					$name = $value;
					$user_info['phone'] = $value;
				}
			}
			if (isset($post_data['web-url'])) {
				$user_info['web-url'] = $post_data['web-url'];
			}
			if (isset($post_data['message'])) {
				$user_info['message'] = $post_data['message'];
			}
			if (isset($post_data['data_to_checkout'])) {
				$user_info['data_to_checkout'] = $post_data['data_to_checkout'];
			}
		}
		$next_step = [];
		switch ($post_action) {
			case 'notification':
				$response['redirect']	= false;
				break;
			case 'redirect_to':
				$action_type 			= 'redirect_to_url';
				$response['redirect']	= true;
				if (!empty($post_data['redirect_url'])) {
					if ($post_data['redirect_url'] !== 'next_step') {
						$action_type 			= 'redirect_to_url';
						$response['redirect_url'] 	= add_query_arg( $this->prepare_query_param($name,$query), $post_data['redirect_url']);
					} elseif ($post_data['redirect_url'] == 'next_step') {
						$action_type 				= 'next_step';

						$next_step 					= $funnel_id ? Wpfnl_functions::get_next_step($funnel_id, $step_id) : '';
						if ($next_step) {
							$response['redirect_url'] 	= add_query_arg( $this->prepare_query_param($name,$query), get_the_permalink($next_step['step_id']));

						} else {
							$response['redirect_url'] = '#';
						}
					}
				} else {
					$response['redirect_url'] = '#';
				}
				break;
			default:
				$action_type 				= 'next_step';
				$next_step 					= $funnel_id ? Wpfnl_functions::get_next_step($funnel_id, $step_id) : '';
				if ($next_step) {
					$response['redirect_url'] 	= add_query_arg( $this->prepare_query_param($name,$query), get_the_permalink($next_step['step_id']));
				} else {
					$response['redirect_url'] = '#';
				}
				$response['redirect']		= true;
		}
		/**
		 * User registration
		 */
		if (!empty($post_data['optin_allow_registration']) && $post_data['optin_allow_registration'] == 'yes') {

			$this->create_user_optin_allow_registration($name, $post_data);
		}


		$admin_email 	 		= isset($post_data['admin_email']) && $post_data['admin_email'] ? $post_data['admin_email'] : false;
		$admin_email_subject 	= isset($post_data['admin_email_subject']) && $post_data['admin_email_subject'] ? $post_data['admin_email_subject'] : 'Opt-in form Submission';

		/**
		 * Recaptcha Check
		 */

		$is_recaptcha 			= isset($post_data['wpf-is-recapcha']) ? $post_data['wpf-is-recapcha'] : 'false';
		$recaptcha_site_key 	= isset($post_data['wpf-optin-g-token']) ? $post_data['wpf-optin-g-token'] : '';
		$recaptcha_site_secrect = isset($post_data['wpf-optin-g-secret-key']) ? $post_data['wpf-optin-g-secret-key'] : '';

		$response = $this->send_mail_after_checking_google_recaptcha_response($is_recaptcha, $recaptcha_site_key, $recaptcha_site_secrect, $admin_email, $admin_email_subject, $user_info, $response);

		$record = (object)[];
		$record->form_data = $user_info;
		do_action('wpfunnels/after_optin_submit', $step_id, $post_action, $action_type, $record);

		if (!empty($next_step['step_type']) && 'conditional' ==  $next_step['step_type']) {
			$next_step = $this->match_conditional_step($next_step, $funnel_id, $step_id );
			$response['redirect_url'] 	= add_query_arg( $this->prepare_query_param($name,$query), get_the_permalink($next_step['step_id']));
		}

		echo json_encode($response, true);
		die();
	}


	/**
	 * Gutenberg opt-in form handling
	 * 
	 * @return void
	 * @since  2.5.9
	 */
	public function gutenberg_optin_form_submission()
	{
		check_ajax_referer('optin_form_nonce', 'security');
		$step_id 	= isset($_POST['step_id']) ? $_POST['step_id'] : '';
		$postData 	= isset($_POST['postData']) ? $_POST['postData'] : '';
		$funnel_id	= Wpfnl_functions::get_funnel_id_from_step($step_id);
		$post_data  = array();
		parse_str($postData, $post_data);

		if (isset($post_data['email'])){
			$post_data['email'] = strtolower($post_data['email']);
		}

		$post_id = $step_id;
		$post = get_post($post_id);
		$all_blocks = parse_blocks($post->post_content);

		$url 	= isset($_POST['url']) ? $_POST['url'] : '';
		$parts = parse_url($url);
		$query = [];
		if( isset($parts['query']) ){
			parse_str($parts['query'], $query);
		}

		$admin_email = '';
		$admin_email_subject = '';
		$post_action = '';
		$block_attr = array();

		$record 			= new Optin_Record($post_data);
		$fields				= $record->get_fields();
		$name				= '';
		if ($fields) {
			foreach ($fields as $key => $value) {
				if ('email' === $key) {
					$name = strstr($value,'@',true);
				} elseif( 'last_name' === $key ) {
					$name = $value;
				} elseif ('first_name' === $key) {
					$name = $value;
				}
			}
		}

		$response 	= array(
			'success'			=> true,
			'post_action' 		=> 'notification',
			'notification_text' => '',
			'redirect_url'		=> '#'
		);
		$custom_url = '';
		$blocks = $this->search_items_by_key($all_blocks, 'blockName');
		foreach ($blocks as $block) {
			if ($block['blockName'] == 'wpfunnels/optin-form') {
				$block_attr = $block['attrs'];

				$admin_email 	= isset($block_attr['adminEmail']) ? $block_attr['adminEmail'] : '';
				$admin_email_subject 	= isset($block_attr['emailSubject']) ? $block_attr['emailSubject'] : 'Opt-in form Submission';

				$post_action 	= isset($block_attr['postAction']) ? $block_attr['postAction'] : 'notification';
				$custom_url 	= isset($block_attr['redirect_url']) ? $block_attr['redirect_url'] : '#';
			}
		}

		$response['notification_text'] = isset($block_attr['notification']) ? $block_attr['notification'] : '';
		$next_step = [];
		switch ($post_action) {
			case 'notification':
				$action_type 			= 'notification';
				$response['redirect']	= false;
				break;
			case 'redirect_to':
				$action_type 			= 'redirect_to_url';
				$response['redirect']	= true;
				$response['post_action'] = $post_action;
				if ($custom_url) {
					$response['redirect_url'] 	= add_query_arg($this->prepare_query_param($name,$query), $custom_url);
				} else {
					$response['redirect_url'] = '#';
				}
				break;
			case 'next_step':
				$response['post_action']	= $post_action;
				$action_type 				= 'next_step';
				$next_step 					= Wpfnl_functions::get_next_step($funnel_id, $step_id);

				$response['redirect_url'] 	= add_query_arg($this->prepare_query_param($name,$query), get_the_permalink($next_step['step_id']));
				$response['redirect']		= true;
				break;
			default:
				$action_type 				= 'next_step';
				$next_step 					= Wpfnl_functions::get_next_step($funnel_id, $step_id);
				$response['redirect_url'] 	= add_query_arg($this->prepare_query_param($name,$query), get_the_permalink($next_step['step_id']));
				$response['redirect']		= true;
		}
		/**
		 * User registration
		 */
		if (!empty($post_data['optin_allow_registration']) && $post_data['optin_allow_registration'] == 'yes') {

			$this->create_user_optin_allow_registration($name, $post_data);
		}
		/**
		 * Check google Recaptcha V3
		 */
		$is_recaptcha 			= isset($post_data['wpf-is-recapcha']) ? $post_data['wpf-is-recapcha'] : 'false';
		$recaptcha_site_key 	= isset($post_data['wpf-optin-g-token']) ? $post_data['wpf-optin-g-token'] : '';
		$recaptcha_site_secrect = isset($post_data['wpf-optin-g-secret-key']) ? $post_data['wpf-optin-g-secret-key'] : '';

		$response = $this->send_mail_after_checking_google_recaptcha_response($is_recaptcha, $recaptcha_site_key, $recaptcha_site_secrect, $admin_email, $admin_email_subject, $post_data, $response);
		do_action('wpfunnels/after_optin_submit', $step_id, $post_action, $action_type, $record);

		if (!empty($next_step['step_type']) && 'conditional' ==  $next_step['step_type']) {
			$next_step = $this->match_conditional_step($next_step, $funnel_id, $step_id );
			$response['redirect_url'] 	= add_query_arg($this->prepare_query_param($name,$query), get_the_permalink($next_step['step_id']));
		}

		echo json_encode($response, true);
		die();
	}


	/**
	 * Search array item by key
	 * 
	 * @param Array $array
	 * @param String $key
	 * 
	 * @return Array
	 * @since  2.5.9
	 */
	private function search_items_by_key($array, $key)
	{
		$results = array();

		if (is_array($array)) {
			if (isset($array[$key]) && key($array) == $key)
				if ($array[$key] == 'wpfunnels/optin-form') {
					$results[] = $array;
				}

			foreach ($array as $sub_array)
				$results = array_merge($results, $this->search_items_by_key($sub_array, $key));
		}

		return  $results;
	}


	/**
	 * Send email to admin
	 * 
	 * @param String $email
	 * @param String $subject
	 * @param Array $user_info
	 * 
	 * @return void
	 * @since  2.5.9
	 */
	private function send_email_to_admin($emails = '', $subject = '', $user_info = [])
	{
		$email_array = explode(",", $emails);
		if (is_array($email_array)) {
			foreach ($email_array as $email) {

				if ($email) {
					$email = trim($email);
					$current_date = date("d M Y");
					$time = new \DateTimeImmutable('now', wp_timezone());
					$current_time = $time->format("h:i A");
					$poweredBy = __('WPFunnels', 'wpfnl');
					$info = '';
					if (isset($user_info['first_name']) && $user_info['first_name']) {
						$info .= "First Name : {$user_info['first_name']}<br>";
					}
					if (isset($user_info['last_name']) && $user_info['last_name']) {
						$info .= "Last Name : {$user_info['last_name']}<br>";
					}
					if (isset($user_info['email']) && $user_info['email']) {
						$info .= "Email : {$user_info['email']}<br><br>";
					}
					if (isset($user_info['phone']) && $user_info['phone']) {
						$info .= "Phone : {$user_info['phone']}<br><br>";
					}
					if (isset($user_info['web-url']) && $user_info['web-url']) {
						$esc_url = esc_url($user_info['web-url']);
						$info .= "Website URL : {$esc_url}<br><br>";
					}
					if (isset($user_info['message']) && $user_info['message']) {
						$esc_html = esc_html($user_info['message']);
						$info .= "Message : {$esc_html}<br><br>";
					}
					$info .= "----<br><br>Date : {$current_date} <br>Time : {$current_time} <br>Powered by : {$poweredBy} <br>";

					$email_body = $info;
					if (!$subject) {
						$subject = 'New Submission';
					}
					$headers = ['Content-Type: text/html; charset=UTF-8'];
					wp_mail($email, $subject, $email_body, $headers);
				}
			}
		}
	}


	/**
	 * Create User as a subscriber
	 * When allow the subscriber option for registration from builder
  	 *
	 * @param $name
	 * @param $post_data
	 * 
	 * @since 2.5.9
	 */
	private function create_user_optin_allow_registration($name, $post_data)
	{
		
		$exist_user_by_username 	= get_user_by('login', $name);
		$exist_user_by_email 		= get_user_by('email', $post_data['email']);
		
		if (empty($exist_user_by_email) || empty($exist_user_by_username)) {
			$data = array(
				'user_login'           	=> $name,
				'user_email'           	=> isset($post_data['email']) ? $post_data['email'] : '',
				'first_name' 			=> isset($post_data['first_name']) ? $post_data['first_name'] : '',
				'last_name' 			=> isset($post_data['last_name']) ? $post_data['last_name'] : '',
				'user_pass'            	=> wp_generate_password(6, false),
				'role'                 	=> 'subscriber',
			);
			if (isset($post_data['web-url'])) {
				$data['user_url'] = $post_data['web-url'];
			}
			$user_id = wp_insert_user($data);
			if (isset($post_data['phone'])) {
				update_user_meta($user_id, 'phone', $post_data['phone']);
			}
			if (!is_wp_error($user_id)) {
				wp_send_new_user_notifications($user_id, $notify = 'both');
			}
		}
	}


	/**
	 * Check Google recaptcha Response and Send mail
     *
	 * @param $is_recaptcha
	 * @param $recaptcha_site_key
	 * @param $recaptcha_site_secrect
	 * 
	 * @since 2.5.9
	 */

	private function send_mail_after_checking_google_recaptcha_response($is_recaptcha, $recaptcha_site_key, $recaptcha_site_secrect, $admin_email, $admin_email_subject, $post_data, $response)
	{
		$recaptcha_response = [];
		if ($recaptcha_site_key &&  $recaptcha_site_secrect && 'on' === $is_recaptcha) {
			$recaptcha_response = Google_Recaptcha_Handler::get_response_recaptcha($recaptcha_site_secrect, $recaptcha_site_key);
		}
		if ('on' === $is_recaptcha) {
			if ($admin_email && $admin_email_subject && isset($post_data['email']) && $post_data['email'] && isset($recaptcha_response->success) && $recaptcha_response->success) {
				$this->send_email_to_admin($admin_email, $admin_email_subject, $post_data);
			} else {
				$response['notification_text'] = __("reCAPTCHA is not Correct", 'wpfnl');
				$response['success'] 			= false;
				return $response;
			}
		} else {
			if ($admin_email && $admin_email_subject && isset($post_data['email']) && $post_data['email']) {
				$this->send_email_to_admin($admin_email, $admin_email_subject, $post_data);
			}
		}
		return  $response;
	}


	/**
	 * Match ption submit condition
     *
	 * @param Array $next_step
	 * @param Number $funnel_id
	 * 
	 * @return Array $next_step
	 * @since  2.5.7
	 */
	private function match_conditional_step($next_step, $funnel_id, $step_id )
	{
		$condition				= Wpfnl_Condition_Checker::getInstance();
		$condition_identifier	= strval($next_step['step_id']);
		$data['step_id']		= $step_id;
		$condition_matched		= $condition->check_condition($funnel_id, $data, $condition_identifier);
		$next_node 				= Wpfnl_functions::get_next_step($funnel_id, $condition_identifier, $condition_matched);
		$next_node       		= apply_filters('wpfunnels/next_step_data', $next_node);
		return $next_node;
	}


	
}
